/*
 * TV-Browser
 * Copyright (C) 04-2003 Martin Oberhauser (darras@users.sourceforge.net)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * CVS information:
 *  $RCSfile: PluginManager.java,v $
 *   $Source: /cvsroot/tvbrowser/tvbrowser/src/devplugin/PluginManager.java,v $
 *     $Date: 2004/04/10 13:22:11 $
 *   $Author: til132 $
 * $Revision: 1.13 $
 */

package devplugin;

import java.util.Iterator;

import util.exc.TvBrowserException;

/**
 * The PluginManager provides some usefull methods for a plugin. 
 * More methods may follow in future versions.
 *
 * @author Martin Oberhauser
 */
public interface PluginManager {

  /**
   * Gets a program.
   * <p>
   * Returns null, if the specified program could not be found.
   */
  public devplugin.Program getProgram(devplugin.Date date, String progID);

  /**
   * Gets the subscribed channels.
   */
  public Channel[] getSubscribedChannels();

  /**
   * Gets an iterator through all programs of the specified channel at the
   * specified date.
   * <p>
   * If the requested data is not available, null is returned.
   */
  public Iterator getChannelDayProgram(devplugin.Date date, Channel channel);

  /**
   * Searches the TV data for programs which match a regular expression.
   *
   * @param regex The regular expression programs must match to.
   * @param inTitle Should be searched in the title?
   * @param inText Should be searched in the desription?
   * @param caseSensitive Should the search be case sensitive?
   * @param channels The channels to search in.
   * @param startDate The date to start the search.
   * @param nrDays The number of days to include after the start date. If
   *        negative the days before the start date are used.
   * @throws TvBrowserException If there is a syntax error in the regular expression.
   * @return The matching programs.
   * 
   * @deprecated Use {@link #search(String, boolean, ProgramFieldType[], Date, int, Channel[], boolean)}
   *             instead.
   */
  public Program[] search(String regex, boolean inTitle, boolean inText,
    boolean caseSensitive, Channel[] channels, devplugin.Date startDate,
    int nrDays)
    throws TvBrowserException;

  /**
   * Searches the TV data base for programs that match a regular expression.
   * 
   * @param regex The regular expression programs must match to.
   * @param caseSensitive Should the search be case sensitive?
   * @param fieldArr The fields to search in
   * @param startDate The date to start the search.
   * @param nrDays The number of days to include after the start date. If
   *        negative the days before the start date are used.
   * @param channels The channels to search in.
   * @param sortByStartTime Should the results be sorted by the start time?
   *        If not, the results will be grouped by date and channel and the
   *        search will be faster.
   * @return The matching programs.
   * @throws TvBrowserException
   * @throws TvBrowserException If there is a syntax error in the regular expression.
   */
  public Program[] search(String regex, boolean caseSensitive,
    ProgramFieldType[] fieldArr, Date startDate, int nrDays, Channel[] channels,
    boolean sortByStartTime)
    throws TvBrowserException;

  /**
   * Returns a list of all installed Plugins
   */
  public Plugin[] getInstalledPlugins();


  /**
   * Returns a TvDataService by class name
   */
  public tvdataservice.TvDataService getDataService(String dataServiceClassName);

  /**
   * Creates a contextmenu containing items for the other plugins.
   */
  public javax.swing.JPopupMenu createPluginContextMenu(final Program program, devplugin.Plugin caller);

  /**
   * Returns an array of all available filters.
   * @since 0.9.7.4
   */
  public ProgramFilter[] getAvailableFilters();
	
  
  /**
   * Returns an example program. You can use it for preview stuff.
   * @since 0.9.7.4
   */
  public Program getExampleProgram();
  
  
  /**
   * @since 0.9.7.4
   */
  public Plugin getDefaultContextMenuPlugin();
  
}