/*
 * TV-Browser
 * Copyright (C) 04-2003 Martin Oberhauser (darras@users.sourceforge.net)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * CVS information:
 *  $RCSfile: AboutBox.java,v $
 *   $Source: /cvsroot/tvbrowser/tvbrowser/src/tvbrowser/ui/aboutbox/AboutBox.java,v $
 *     $Date: 2004/08/22 20:48:34 $
 *   $Author: darras $
 * $Revision: 1.10 $
 */
 
package tvbrowser.ui.aboutbox;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Frame;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.datatransfer.StringSelection;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JEditorPane;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;

import tvbrowser.TVBrowser;
import util.ui.ImageUtilities;
import util.ui.html.ExtendedHTMLDocument;
import util.ui.html.ExtendedHTMLEditorKit;

/**
 * 
 * @author Martin Oberhauser (darras@users.sourceforge.net)
 *
 */
public class AboutBox extends JDialog {

  private static final util.ui.Localizer mLocalizer
    = util.ui.Localizer.getLocalizerFor(AboutBox.class);
  
  private static Font smallFont=new Font("Dialog", Font.PLAIN, 10);
  private static Font bigFont=new Font("Dialog", Font.PLAIN, 24);
  private static Font normalFont=new Font("Dialog", Font.PLAIN, 12);
  private static Font boldFont=new Font("Dialog",Font.BOLD, 12);
  

  public AboutBox(Frame parent) {
    super(parent,true);

    setTitle(mLocalizer.msg("about", "About"));
    
    JPanel contentPane=(JPanel)getContentPane();
    contentPane.setBackground(Color.WHITE);
    
    contentPane.setLayout(new BorderLayout());

    
    JPanel right = new JPanel();
    right.setLayout(new BorderLayout());
    right.setBackground(Color.WHITE);
    
    final JEditorPane infoEP = new JEditorPane();
    infoEP.setOpaque(false);
    infoEP.setEditorKit(new ExtendedHTMLEditorKit());
    ExtendedHTMLDocument doc = (ExtendedHTMLDocument) infoEP.getDocument();
    String text = createAboutText(doc);
    infoEP.setText(text);
    infoEP.setEditable(false);
    
    right.add(infoEP,BorderLayout.CENTER);


    JPanel btnPanel=new JPanel(new FlowLayout(FlowLayout.RIGHT));
    btnPanel.setBorder(BorderFactory.createEmptyBorder(0, 0, 10, 10));
    btnPanel.setBackground(Color.WHITE);

    JButton copyClipboard = new JButton(mLocalizer.msg("copyClipboard", "Copy to Clipboard"));
    copyClipboard.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
            Document doc = infoEP.getDocument();

            try {
                StringSelection sel = new StringSelection(doc.getText(0, doc.getLength()));
                Toolkit.getDefaultToolkit().getSystemClipboard().setContents(sel, null);
            } catch (BadLocationException e1) {
                e1.printStackTrace();
            }
        }
      });
    btnPanel.add(copyClipboard);
    
    JButton closeBtn = new JButton(mLocalizer.msg("close", "Close"));
    final JDialog parentFrame=this;

    closeBtn.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        parentFrame.hide();
      }
    });
    
    getRootPane().setDefaultButton(closeBtn);
    btnPanel.add(closeBtn);
    right.add(btnPanel,BorderLayout.SOUTH);
    
    contentPane.add(right, BorderLayout.CENTER);
    
    Image image = ImageUtilities.createImage("imgs/tvabout.jpg");
    if (image != null) {
      ImageUtilities.waitForImageData(image, null);
      JLabel gfx = new JLabel(new ImageIcon(image));
      
      JPanel gfxPanel = new JPanel(new BorderLayout());
      gfxPanel.setBackground(Color.WHITE);
      gfxPanel.add(gfx, BorderLayout.SOUTH);
      
      contentPane.add(gfxPanel, BorderLayout.WEST);
    }
  }
  
  private StringBuffer createInfoEntry(StringBuffer buf, String key, String value) {
    
    buf.append("<tr><td width=\"35%\" valign=\"top\">");
    buf.append("<div id=\"key\">");
    
    buf.append(key);
    buf.append("</div>");
    return buf.append("</td><td>").append(value).append("</td></tr>");
  }
  
  private void createSpacer(StringBuffer buf) {
      buf.append("<tr><td id=\"small\"></td></tr>");
      
  }

  private void createJavaVersionEntry(StringBuffer buf) {
      
      buf.append("<tr><td colspan=\"2\">");
      
      buf.append("<div id=\"small\">");

      buf.append(System.getProperty("java.vm.name") + "<br>");
      buf.append(System.getProperty("java.vendor") + "<br>");
      buf.append(System.getProperty("java.home"));
      
      buf.append("</div>");
      
      buf.append("</td></tr>");
  }
  
  private String createAboutText(ExtendedHTMLDocument doc) {
  
    StringBuffer buf=new StringBuffer();
    buf.append("<html>" +
               "  <head>" +
               "<style type=\"text/css\" media=\"screen\">" +
               "<!--" +
               "body {font-family:Dialog;}" +
               "" +
               "#title { font-size:18px; font-family:Dialog; text-align:center; font-weight:bold; margin-top:5px}" +
               "#key { font-size:12px; font-family:Dialog; font-weight:bold; }" +
               "" +
               "#small { font-size:9px; font-family:Dialog; margin-bottom: 5px}" +
               "-->" +
               "  </head>" +
               "  <body>" +
               "    <div id=\"title\">"+mLocalizer.msg("version", "Version")+": " + TVBrowser.VERSION.toString() +"</div>" +
               "<p>" +
               "    <table width=\"100%\"");
     
    
    createInfoEntry(buf, mLocalizer.msg("platform", "Platform") + ":",
       System.getProperty("os.name") + " " + System.getProperty("os.version"));
       
    createInfoEntry(buf, mLocalizer.msg("system", "System") + ":",
       System.getProperty("os.arch"));
    
    createSpacer(buf);
    
    createInfoEntry(buf, mLocalizer.msg("javaVersion", "Java Version") + ":",
       System.getProperty("java.version"));
    
    createJavaVersionEntry(buf);
    
    createSpacer(buf);
    
    createInfoEntry(buf, mLocalizer.msg("location", "Location") + ":",
    System.getProperty("user.country") + "," + System.getProperty("user.language"));
               
    java.util.TimeZone timezone = java.util.TimeZone.getDefault();
    int tzOffset = timezone.getRawOffset() / 1000 / 60 / 60;
    String tzOffsetAsString = mLocalizer.msg("hours", "({0,number,+#;#} hours)",
    new Integer(tzOffset));           
               
    createInfoEntry(buf, mLocalizer.msg("timezone", "Timezone") + ":",
       timezone.getDisplayName() + " " + tzOffsetAsString);         
                
               
    buf.append("</table></p>");
    buf.append("<p>");
    buf.append("<div id=\"small\">");
    
    buf.append(mLocalizer.msg("copyrightText",
          "Copyright (c) 04/2003 by Martin Oberhauser, Til Schneider, Bodo Tasche, under the"
          + "GNU General Public License"));
          
    buf.append("</div>");
    
    buf.append("<div id=\"small\">");
    buf.append("This product includes software developed " +
                       "by L2FProd.com (http://www.L2FProd.com/).");
    
    buf.append("</div>");
    buf.append("</p>");
    buf.append("  </body>" +
               "</html>");
    return buf.toString();
  }
}
